<?php

/**
 * Class jsonFieldset
 */
class jsonFieldset {

  /**
   * @var
   */
  private $fieldset;

  /**
   * @var
   */
  private $fields;

  /**
   * @var mixed
   */
  private $field;

  /**
   * @var
   */
  private $parsedCollection;

  /**
   * @var
   */
  private $jsonCollection;


  /**
   * Loop through all of the instances of the json_fieldset field and decode
   * each one.
   *
   * @param $fieldset
   */
  function __construct($fieldset) {

    if(is_array($fieldset)) {
      foreach ($fieldset as $set) {
        $this->fields[] = $this->_decodeJsonField($set['fieldset']);
      }
      return $this;
    }

    // If we are handed a json string for a single fieldset.
    if($fieldset != '') {
      $this->field = json_decode($fieldset, TRUE);
      return $this;
    }

  }

  /**
   * Return the json decoded field for the default values in the form when
   * editing the content. No parsing is needed here.
   *
   * @return mixed
   */
  public function formFieldValues() {
    return $this->field;
  }

  /**
   * Loop through the encoded field instances and parse the field into the
   * indexes and values we want.
   *
   * @return $this
   */
  public function parseFields() {

    foreach($this->fields as $values) {
      $this->parsedCollection[] = $this->_parseFields($values);
    }
    // Make a json encoded string available as well.
    $this->jsonCollection = json_encode($this->parsedCollection);
    return $this;
  }

  /**
   * @return mixed
   */
  public function parseField() {
    $this->parsedCollection = $this->_parseField($this->field);
    return $this;
  }

  /**
   * Return the parsed collection array.
   *
   * @return mixed
   */
  public function toArray(){
    return $this->parsedCollection;
  }

  /**
   * Return the json encoded collection.
   *
   * @return mixed
   */
  public function toJson(){
    return $this->jsonCollection;
  }

  /**
   * Decode the json string into an associative array.
   *
   * @param $field
   * @return mixed
   */
  private function _decodeJsonField($field) {
    return json_decode($field, TRUE);
  }

  /**
   * @param $field
   * @return mixed
   */
  private function _parseField($field) {
    foreach($field as $key => $value) {
      $method = $this->_fieldMap($value['type']);
      $function = '_parse' . $method;
      $parsed[$key] = $this->{$function}($value);
    }
    return $parsed;
  }

  /**
   * Build the array for the field by getting its method by field type.
   *
   * @param $values
   * @return mixed
   */
  private function _parseFields($values) {

    foreach($values as $key => $value) {
      $method = $this->_fieldMap($value['type']);
      $function = '_parse' . $method;
      $field[$key] = $this->{$function}($value);
    }

    return $field;

  }

  /**
   * Parse a textarea.
   *
   * @param $field
   * @return mixed
   */
  private function _parseTextarea($field) {
    $field['type'] = $field['type'];
    $field['value'] = $field['value'];
    return $field;
  }

  /**
   * Parse a managed file.
   *
   * @param $field
   * @return mixed
   */
  private function _parseManagedFile($field) {
    $field['type'] = $field['type'];
    $field['value'] = $field['value'];
    $field['uri'] = image_style_url('large', file_load($field['value'])->uri);
    return $field;
  }

  /**
   * Parse a textfield.
   *
   * @param $field
   * @return mixed
   */
  private function _parseTextField($field) {
    $field['type'] = $field['type'];
    $field['value'] = $field['value'];
    return $field;
  }

  /**
   * Creates map of fields for respective class names to be used to manage
   * the fields based on the field type.
   *
   * @param $type
   * @return string
   */
  private function _fieldMap($type) {

    $method = '';

    switch ($type) {

      case 'textfield' :
        $method = 'TextField';
        break;

      case 'managed_file':
        $method = 'ManagedFile';
        break;

      case 'textarea':
        $method = 'Textarea';
        break;

    }

    return $method;

  }

}